SET QUOTED_IDENTIFIER ON 
GO
SET ANSI_NULLS ON 
GO

ALTER PROCEDURE [dbo].[SaveEvaluationBaseResult] (
	@VCC VARCHAR(50) = NULL,
	@EVALUATION_PLAN_ID VARCHAR(100) = NULL,
	@ABILITY_TYPE VARCHAR(50) = NULL
) AS

BEGIN
	-- ALERTA: NO ESTA PROBBADO PARA PLAN MANUAL

	-- ESTE SP INSERTA LOS RESULT BASE DE UN PLAN AUTOMATICO/MIXTO
	SET NOCOUNT ON

	-- SETTEAMOS EL TYPE ABILITY COMO AUTOMATIC PO DEFECTO
	IF @ABILITY_TYPE IS NULL SET @ABILITY_TYPE = 'AUTOMATIC'

	-- EMULAMOS UNA NUEVA TABLA VARIABLE PARA OBTENER LOS REGISTROS UNICOS QUE INSERTAREMOS EN LA TABLA EVALUATION_RESULT
	--  ORDENANDO POR SECTION Y PART

	DROP TABLE IF EXISTS #TMP_TRANSCRIBED_CALL_INTERACTIONS
	DROP TABLE IF EXISTS #TMP_RESULTS_FOR_EVALUATION	

	-- CREAMOS UNA TABLA TEMPORAL PARA ALMACENAR LAS INTERACCIONES CALL YA TRANSRITAS
	SELECT A.[ID]
		INTO #TMP_TRANSCRIBED_CALL_INTERACTIONS
		FROM [dbo].[Audio] A (NOLOCK)
		INNER JOIN [dbo].[ProcessAudio] PA (NOLOCK)
			ON PA.[VCC] = A.[VCC]
				AND PA.[AudioId] = A.[ID] 
		INNER JOIN [dbo].[EvaluationPlan] EP (NOLOCK)
			ON EP.[Instance] = A.[VCC]
				AND EP.[ProcessId] = PA.[ProcessId]
		WHERE A.[VCC] = @VCC 
			AND A.[IsTranscribed] = 1
			AND EP.[ID] = @EVALUATION_PLAN_ID

	-- CREAMOS UNA TABLA TEMPORAL PARA ALMACENAR LAS INTERACCIONES TEXT YA TRANSRITAS
	SELECT AI.[ID]
		INTO #TMP_TRANSCRIBED_TEXT_INTERACTIONS
		FROM [dbo].[AsyncInteraction] AI (NOLOCK)
		INNER JOIN [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
			ON IEP.[Instance] = AI.[VCC]
				AND IEP.[IdInteraction] = AI.[ID]
				AND IEP.[FormattedEndDate] = AI.[Folder]
		WHERE AI.[VCC] = @VCC 
			AND AI.[IsTranscribed] = 1
			AND IEP.[IdInteraction] = @EVALUATION_PLAN_ID

	-- GENERAMOS REGISTROS PARA TODAS LAS INTERACCIONES
	;WITH RAW_CTE AS (
		SELECT IEP.[Instance]									'vcc',
				IEP.[IdCampaign]								'IdCampaign',
				IEP.[IdInteraction]								'IdInteraction',
				IEP.[IdAgent]									'IdAgent',
				EP.[IdEvaluationFormat]							'IdEvaluationFormat',
				EP.[ProcessId]									'ProcessId',
				HEF.[MinQualificationAcceptable]				'MinQualificationAcceptable',
				HAEF.[IdGroup]									'IdGroup',
				HAEF.[IdSubGroup]								'IdSubGroup',
				HAEF.[IdAbility]								'IdAbility',
				HAEF.[Score]									'MaxAbilityScore',
				HS.[Value]										'AbilityInnerScore',
				EP.[ID]											'IdEvaluationPlan',
				HS.[Description]								'ScoreDescription',
				IEP.[IdCampaign]								'CampaignDescription',
				IEP.[IdAgent]									'AgentDescription',
				IEP.[InteractionDateStart]						'InteractionDateini',
				IEP.[InteractionDateEnd]						'InteractionDatefin',
				IEP.[ContactId]									'InteractionAddress',
				HAEF.[Order]									'AbilityOrder',
				IEP.[InteractionType]							'InteractionType',
				HAEF.[ID]										'IdAbilityEvaluationFormat',
				HAEF.[Conditional]								'Conditional',
				RN = ROW_NUMBER() OVER(PARTITION BY IEP.[IdCampaign],
													IEP.[IdInteraction],
													IEP.[IdAgent],
													EP.[IdEvaluationFormat],
													HEF.[MinQualificationAcceptable],
													HAEF.[IdGroup],
													HAEF.[IdSubGroup],
													HAEF.[IdAbility],
													HS.[Value],
													HS.[Description],
													IEP.[IdCampaign],
													IEP.[IdAgent],
													IEP.[InteractionDateStart],
													IEP.[InteractionDateEnd],
													HAEF.[Order],
													IEP.[InteractionType],
													HAEF.[ID],
													HAEF.[Conditional]
										ORDER BY IEP.[InteractionSection] ASC, IEP.[InteractionPart] ASC)
			FROM [dbo].[EvaluationPlan] EP (NOLOCK)
			INNER JOIN [dbo].[HistoricEvaluationFormat] HEF (NOLOCK)
				ON HEF.[VCC] = EP.[Instance]
					AND HEF.[EvaluationPlanId] = EP.[ID]
			INNER JOIN [dbo].[InteractionEvaluationPlan] IEP (NOLOCK)
				ON IEP.[Instance] = EP.[Instance]
					AND IEP.[IdEvaluationPlan] = EP.[ID]
			INNER JOIN [dbo].[HistoricAbilityEvaluationFormat] HAEF (NOLOCK)
				ON HAEF.[VCC] = EP.[Instance]
					AND HAEF.[EvaluationPlanId] = EP.[ID]
			INNER JOIN [dbo].[HistoricAbility] HA (NOLOCK)
				ON HA.[VCC] = EP.[Instance]
					AND HA.[EvaluationPlanId] = EP.[ID]
			INNER JOIN [dbo].[HistoricScore] HS (NOLOCK)
				ON HS.[VCC] = IEP.[Instance]
					AND HS.[EvaluationPlanId] = EP.[ID]
					AND HS.[IdAbility] = HA.[ID]
					AND HS.[Value] = IIF(HA.[IsNegative] = 1, 100, 0)
			LEFT JOIN [dbo].[EvaluationResult] ER (NOLOCK)
				ON ER.[Instance] = IEP.[Instance]
					AND ER.[IdEvaluationPlan] = IEP.[IdEvaluationPlan]
					AND ER.[IdAgent] = IEP.[IdAgent]
					AND ER.[IdAbilityEvaluationFormat] = HAEF.[ID]
			WHERE ER.[Instance] IS NULL
				AND IEP.[IdInteraction] IS NOT NULL
				AND EP.[Instance] = @VCC
			    AND EP.[ID] = @EVALUATION_PLAN_ID
				AND (
					(@ABILITY_TYPE = 'AUTOMATIC' AND HA.[Type] = 'MANUAL_AUTOMATIC')
					OR
					(@ABILITY_TYPE = 'MANUAL' AND HA.[Type] = 'MANUAL')
				)
				
	) --select * from RAW_CTE
	-- FILTRAMOS SOLO LA PRIMERA OCURRENCIA
	, FILTERED_CTE AS (
		SELECT C.* 
			FROM RAW_CTE C
			WHERE C.[RN] = 1
	) --select * from FILTERED_CTE

	-- INSERTAMOS TODOS LOS REGISTROS EN UNA TABLA TEMPORAL
	SELECT * 
		INTO #TMP_RESULTS_FOR_EVALUATION
		FROM FILTERED_CTE

	-- ELIMINAMOS LOS REGISTROS PARA INTERACCIONES CALL NO TRANSCRITAS
	DELETE FROM #TMP_RESULTS_FOR_EVALUATION
		WHERE [InteractionType] = 'CALL' 
			AND [IdInteraction] NOT IN (
				SELECT [ID] FROM #TMP_TRANSCRIBED_CALL_INTERACTIONS
			)

	-- ELIMINAMOS LOS REGISTROS PARA INTERACCIONES TEXT NO TRANSCRITAS
	/*
	DELETE FROM TMP_RESULTS_FOR_EVALUATION
		WHERE [InteractionType] <> 'CALL' 
			AND [IdInteraction] NOT IN (
				SELECT [ID] FROM #TMP_NOT_TRANSCRIBED_TEXT_INTERACTIONS
			)
			*/

	-- TENEMOS LOS REGISTROS PARA INSERTAR EN EVALUATIONRESULT
	-- SELECT * FROM TMP_RESULTS_FOR_EVALUATION

	-- INSERTAMOS SOLO LAS PRIMERAS OCURRENCIAS PARA CADA INTERACCION
	INSERT INTO [dbo].[EvaluationResult] (
		[Instance], [IdCampaign], [IdInteraction],
		[IdAgent], [IdEvaluationFormat], [MinQualificationAcceptable],
		[IdGroup], [IdSubGroup], [IdAbility],
		[Qualification], [OwnScore], [AbilityInnerScore],
		[IdEvaluator], [EvaluationDate], [MaxAbilityScore],
		[IdEvaluationPlan], [CreatedDate], [CreatedByUserId],
		[LastModifiedDate], [LastModifiedByUserId], [TimeStamp],
		[Comment], [NotApplies], [ScoreDescription],
		[CampaignDescription], [AgentDescription], [InteractionDateini],
		[InteractionDatefin], [InteractionAddress], [AbilityOrder],
		[InteractionType], [IdAbilityEvaluationFormat],  [IdAuditor],
		[Conditional]
	)
		SELECT TMP_RFE.[vcc], TMP_RFE.[IdCampaign], TMP_RFE.[IdInteraction],
				TMP_RFE.[IdAgent], TMP_RFE.[IdEvaluationFormat], TMP_RFE.[MinQualificationAcceptable],
				TMP_RFE.[IdGroup], TMP_RFE.[IdSubGroup], TMP_RFE.[IdAbility],
				(TMP_RFE.[MaxAbilityScore] * TMP_RFE.[AbilityInnerScore] / 100), (TMP_RFE.[MaxAbilityScore] * TMP_RFE.[AbilityInnerScore] / 100), TMP_RFE.[AbilityInnerScore],
				'SPEECH', GETUTCDATE(), TMP_RFE.[MaxAbilityScore],
				TMP_RFE.[IdEvaluationPlan], GETUTCDATE(), 'SPEECH',
				--'100', TMP_RFE.[IdEvaluationPlan], GETUTCDATE(),
				GETUTCDATE(), 'SPEECH', GETUTCDATE(),
				NULL, '0', TMP_RFE.[ScoreDescription],
				TMP_RFE.[CampaignDescription], TMP_RFE.[AgentDescription], TMP_RFE.[InteractionDateini],
				TMP_RFE.[InteractionDatefin], TMP_RFE.[InteractionAddress], TMP_RFE.[AbilityOrder],
				TMP_RFE.[InteractionType], TMP_RFE.[IdAbilityEvaluationFormat], 'SPEECH',
				TMP_RFE.[Conditional]
			FROM #TMP_RESULTS_FOR_EVALUATION TMP_RFE

	-- ELIMINAMOS LAS TABLAS TEMPORALES
	DROP TABLE #TMP_TRANSCRIBED_CALL_INTERACTIONS
	DROP TABLE #TMP_RESULTS_FOR_EVALUATION

	SET NOCOUNT OFF

END
